/*
 * module1.c
 * Basic demo program for demonstrating the use of the HERON-API to 
 * communicate between two processors..
 * Before running this program, take a look at the 'network' file, 
 * it contains information describing the actual hardware configuration
 * and should be updated before running the server.
 *
 *     PW    ver 2.0    30-3-00   changed for heron_api v2.3
 *     JT    ver 2.1  19-7-00    changed for server/loader 3.2
 */

/* YOU need to set this value to reflect how your module is connected
to the next module. */
#define FIFONO	2	/* FIFO through which module communicates with next */

#include <stdioc60.h>
#include <string.h>
#include "heron4_pc8.h"
#include <std.h>
#include <swi.h>

/* DSP/BIOS treats main() as an initialisation routine. While main() */
/* is being executed, DSP/BIOS is initialised but not running: all   */
/* hardware and software interrupts are disabled. This is a good     */
/* place to run bootloader(), since no other threads/tasks will be   */
/* running; tasks or threads that may concurrently try to communica- */
/* with processors that have not yet been booted.                    */

main()
{
	/* 
	 * The next line insures that the program is properly loaded onto
	 * the root DSP. The bootloader() function will call an external
	 * function "init_io_functions". This function is implemented in 
	 * a file "stub.c" in this directory. Please include this file
	 * into your project, and make sure that the correct heron.h
	 * file is included in this "stub.c" file .
	 */
	bootloader();
}

void maintask(void)
{
	HeronFIFO *rfifo;
	HeronFIFO *wfifo;
	int status;
	int data;
	int pattern[32];
	int results[32];

	int processors = 2;
	int i;
	int errors=0;
	/*
	 * All processors in the system are required to clear the config line
	 * to an in-active state. One all processors in a system performed a
	 * HeronConfigOff(), the config line goes in-active, and GDIO modules
	 * are only then able to send data onto C6xxx modules. 
	 * If you only use processor modules but no GDIO modules, then this
	 * doesn't matter to you and you can leave the config line in any
	 * state (active -> don't do a HeronConfigOff() or in-active -> do a
	 * HeronConfigOff() 
	 */
	HeronConfigOff();

	printf(" first HERON module has started running\n");

/* open the FIFO for reading*/
    rfifo=HeronOpenFifo(FIFONO,"r");
    if (rfifo==NULL)
     {
     printf("Cannot open fifo %d for read. Error %d\n",FIFONO,heronerr);
     srv_exit(-1); 	       
     }
/* open the FIFO for writing*/
    wfifo=HeronOpenFifo(FIFONO,"w");
    if (wfifo==NULL)
     {
     printf("Cannot open fifo %d for write. Error %d\n",FIFONO,heronerr);
     srv_exit(-1); 	
     } 

	while (processors)
	{
        /* Try to read the message from the next HERON module */
  	status = HeronRead(rfifo,&data,1);
	if (status == HERON_IO_IN_PROGRESS)
	{
	status = HeronWaitIo(rfifo);
	}
	if (status != HERON_OK)
	{
	printf("Cannot read fifo %d. Error %d\n",FIFONO,heronerr);
	srv_exit(-1);
	}

	printf("HERON module id %x has started replied\n", data);
	processors--;
	}

	errors = 0;


	for (i=0; i<32; i++)
	{
	pattern[i]=1<<i;
	results[i]=0;
	}

/* send the pattern to the next module */
  	status = HeronWrite(wfifo,pattern,32);
	if (status == HERON_IO_IN_PROGRESS)
	{
	status = HeronWaitIo(wfifo);
	}
	if (status != HERON_OK)
	{
	printf("Cannot write fifo %d. Error %d\n",FIFONO,heronerr);
	srv_exit(-1);
	}
/* read the reply from the next HERON module */
  	status = HeronRead(rfifo,results,32);
	if (status == HERON_IO_IN_PROGRESS)
	{
	status = HeronWaitIo(rfifo);
	}
	if (status != HERON_OK)
	{
	printf("Cannot read fifo %d. Error %d\n",FIFONO,heronerr);
	srv_exit(-1);
	}
/* check the data */
	for (i=0;i<32;i++)
	{
	if (results[i] != (1<<i)) errors++;
	}

	if (errors)
	{
		printf("%d Errors in bitwise echo test.\n", errors);
	}
	else
	{
		printf("Bitwise echo test completed successfully.\n");
	}

/* close the FIFOs */
	status = HeronClose(rfifo);
	if (status != HERON_OK)
	{
	printf(" Error closing read fifo \n");
        srv_exit(0); /*HUNT ENGINEERING Server/Loader exit */
	}
	status = HeronClose(wfifo);
	if (status != HERON_OK)
	{
	printf(" Error closing write fifo \n");
        srv_exit(0); /*HUNT ENGINEERING Server/Loader exit */
	}



	/*
	 * 'srv_exit()' will also exit from the server.
	 */
	srv_exit( 0);
}
/*** End Of File ***/
