/*
 * test.c
 * Basic test program to demonstrate a host program that uses the Server and Host API.
 * Before running this program, take a look at the 'network' file, 
 * it contains information describing the actual hardware configuration
 * and should be updated before running the server.
 *
 *     PW    ver 2.0  30-3-00    changed for heron_api v2.3
 *     JT    ver 2.1  19-7-00    changed for server/loader 3.2
 */

#include <stdioc60.h>
#include <string.h>
#include <std.h>
#include <swi.h>

#include <heron1_pc8.h>
/* YOU need to set this value to reflect how your module is connected
to your HOST machine. */
#define FIFONO	0	/* FIFO through which module communicates with host */

extern far SWI_Obj SWI0;

void flash_config(void)
{
	int i, j;

	for (i=0; i < 10; i++)
	{
		HeronConfigOn();
		j=0;
		while (j<1000000) j++;
		HeronConfigOff();
		j=0;
		while (j<1000000) j++;
	}
}

/* DSP/BIOS treats main() as an initialisation routine. While main() */
/* is being executed, DSP/BIOS is initialised but not running: all   */
/* hardware and software interrupts are disabled. This is a good     */
/* place to run bootloader(), since no other threads/tasks will be   */
/* running; tasks or threads that may concurrently try to communica- */
/* with processors that have not yet been booted.                    */

main()
{
	/* 
	 * The next line insures that the program is properly loaded onto
	 * the root DSP. The bootloader() function will call an external
	 * function "init_io_functions". This function is implemented in 
	 * a file "stub.c" in this directory. Please include this file
	 * into your project, and make sure that the correct heron.h
	 * file is included in this "stub.c" file .
	 */
	bootloader();
}

void maintask(void)
{
	HeronFIFO *rfifo;
	HeronFIFO *wfifo;
	int status;
	int data;

	/* If you run "net.serfit()" on the host side, you can execute stdio calls  */
	/* until you call srv_exit(). This function tells the Server/Loader to stop */
	/* serving the system. When you don't run net.serfit() on the host side,    */
	/* you cannot run stdio functions - thus the printf() and srv_exit() below  */
	/* cannot be executed. In that case, comment out those lines and recompile. */
#if WITH_SERVER
	printf("\nServer/Loader running.\n");

	/*
	 * 'srv_exit()' will also exit from the server.
	 */
	quit_server();
#endif

/*initialise digital I/Os*/
HeronDigioOut(0x0);

/* open the FIFO for reading*/
    rfifo=HeronOpenFifo(FIFONO,"r");
    if (rfifo==NULL)
     {
     HeronDigioOut(0x1); /* assert digital I/O bit 0 to show the error*/
     exit(-1); 	       
     }
/* open the FIFO for writing*/
    wfifo=HeronOpenFifo(FIFONO,"w");
    if (wfifo==NULL)
     {
     HeronDigioOut(0x2); /* assert digital I/O bit 1 to show the error*/
     exit(-1); 	
     } 

/* read a word from the host program */
  	status = HeronRead(rfifo,&data,1);
	if (status == HERON_IO_IN_PROGRESS)
	{
	status = HeronWaitIo(rfifo);
	}
        if (status != HERON_OK) 
	{
	HeronDigioOut(0x4); /* assert digital I/O bit 2 to show the error*/
	exit(-1);
	}

	if (data==0x11223344) flash_config();

	/* OK, now the API has started running. Do something. */
	
/* write a word to the host program */
	data = 0xabcdef;
  	status = HeronWrite(wfifo,&data,1);
	if (status == HERON_IO_IN_PROGRESS)
	{
	status = HeronWaitIo(wfifo);
	}
        if (status != HERON_OK) 
	{
	HeronDigioOut(0x8); /* assert digital I/O bit 3 to show the error*/
	exit(-1);
	}

exit(0);
}
/*** End Of File ***/
