
#if (WIN32 && CCSTUDIO)
#include <afxdisp.h>
#endif

#ifdef _LINUX
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#else
#ifdef _MSDOS
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#else
#ifdef _RTOS32
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <rttarget.h>
#include <rttbios.h>
#include <rtk32.h>
#include <rtkeybrd.h>
extern "C" { char getch(void); }
#else
#ifdef BL    // Borland
#include <vcl\vcl.h>
#pragma hdrstop
#else
//#include <stdafx.h>
#endif
#endif
#endif
#endif

#include "signal.h"

#include "heapi.h"
#include "network.h"

#if (WIN32 && CCSTUDIO)
#include "ccif.h"
#endif

#define VERSION		"3.2"

void ShowVersion()
{
	printf( "My Server/Loader %s\n", VERSION);
}

void print_usage(char *name)
{
	ShowVersion();
    fprintf(stderr, "Usage is \"%s [-rsvlifp] <network_description_file>\"\n", name);
    fprintf(stderr, "Options:  -r		Reset the target processors.\n");
    fprintf(stderr, "          -l		Load the target  processors.\n");
    fprintf(stderr, "          -s 		After Reset/Load go to server mode.\n");
    fprintf(stderr, "          -v		Verbose mode.\n");
    fprintf(stderr, "          -v<=n>	Verbose mode, n selects what detail to view.\n");
	fprintf(stderr, "			n=2: view routing tables on nodes.\n");
	fprintf(stderr, "          -c<=n>	Info level when viewing IDROM/EEPROM info\n");
    fprintf(stderr, "          -i<path> 	Indicate directories to look for init routines.\n");
	fprintf(stderr, "          -k		Use packet mode with packet size 256.\n");
	fprintf(stderr, "          -k<=n>	Use packet mode with packet size n.\n");
    fprintf(stderr, "          -d 		Use Dual Comports (C4x only)\n");
	fprintf(stderr, "          -x 		Don't match Module ID with Code Composer Studio ID.\n");
	fprintf(stderr, "          -w<=n>   When writing to the board, wait at most n milliseconds.\n");
#if _VXW_
    fprintf(stderr, "          -m 		Master Mode off (PCI cards)\n");
    fprintf(stderr, "          -n 		Don't use interrupts.\n");
    fprintf(stderr, "          -q<irq> 	Use interrupt <irq> (must be 10,11,12 or 15).\n");
#else
	fprintf(stderr, "          -g		Debug Switch (use Code Composer Studio)\n");
	fprintf(stderr, "       or -g=<n>	Wait factor, decoupling time between Server/Loader\n");
	fprintf(stderr, "			and Code Composer, in seconds.\n");
#endif
	fprintf(stderr, "          -a       Skip FPGA programming, if declared in network file\n");
	fprintf(stderr, "          -t       Use .out file in .\\debug (instead of .out in .\\release\n");
	fprintf(stderr, "          -j       Use bootjumpers to boot HERON modules (HEPC9 only)\n");
}

void add_include_path(network *n, char *s)
{
	char *err;

	err = n->AddIncludePath(s);
	/* Return value is pointer to error message! */
	if (err != NULL)
	{
		fprintf(stderr, "%s\n", err);
		exit   (0);
	}
}

void parse_command_line(network *n, int argc, char *argv[])
{
	int 	i, l, f, ps, wp, t=0;
	char	*s;
	int		filename_ok	= FALSE;
	int		skip        = FALSE;

	if (argc==1)
	{
		print_usage(argv[0]);
#ifdef _RTOS32
		getch();
#endif
		exit(0);
	}

	for (i=1; i<argc; i++)
	{
		s = argv[i];

		skip = FALSE;
		switch(*s)
		{
			case '-':	/* It is an option */
				while (( *s)&&( skip==FALSE))
				{
					switch(*++s)
					{
					case 'r': case 'R':	/* Reset ------- */ n->SetResetSwitch(TRUE);	break;

					case 's': case 'S':	/* Serve ------- */ n->SetServeSwitch(TRUE);	break;

					case 'v': case 'V':	/* Verbose ----- */ n->SetVerbose(TRUE);
							if (s[1] == '=')
							{
								s += 2; /* point to number string */
								l  = 0;
								if ((*s < '0') || (*s > '9'))
								{
									char mess[256];
									sprintf  (mess,"Expected a number after \"v=\"\n");
									n->eprint(mess);
									exit(0);
								}
								while ((*s >= '0') && (*s <= '9'))
								{
									l = l*10 + (*s-'0');
									s++;
								}
								s--;
								n->SetVerbose(l);
							}
							break;


					case 'l': case 'L':	/* Load -------- */ n->SetLoadSwitch(TRUE);		break;

					case 'd': case 'D':	/* Dual Comports */	n->SetDualSwitch(TRUE);		break;

					case 'c': case 'C': /* Show IDROM -- */
							n->SetInfoLevel(1);
							if (s[1] == '=')
							{
								s += 2; /* point to number string */
								l  = 0;
								if ((*s < '0') || (*s > '9'))
								{
									char mess[256];
									sprintf  (mess,"Expected a number after \"c=\"\n");
									n->eprint(mess);
									exit(0);
								}
								while ((*s >= '0') && (*s <= '9'))
								{
									l = l*10 + (*s-'0');
									s++;
								}
								s--;
								n->SetInfoLevel(l);
							}
							break;

					case 'i': case 'I':	/* Add a scan directory */
							++s;
							add_include_path(n, s);
							skip = TRUE;
						break;

					case 'k': case 'K': /* use packets */
							n->SetUsePackets(1);
							ps = 1024;	/* default */
							if (s[1] == '=')
							{
								s += 2; /* point to number string */
								ps = 0;
								if ((*s < '0') || (*s > '9'))
								{
									char mess[256];
									sprintf  (mess,"Expected a number after \"k=\"\n");
									n->eprint(mess);
									exit(0);
								}
								while ((*s >= '0') && (*s <= '9'))
								{
									ps = ps*10 + (*s-'0');
									s++;
								}
								s--;
							}
							n->SetPacketSize(ps);
							break;

					case 'w': case 'W': /* use packets */
							wp = 0;	/* default */
							if (s[1] == '=')
							{
								s += 2; /* point to number string */
								wp = 0;
								if ((*s < '0') || (*s > '9'))
								{
									char mess[256];
									sprintf  (mess,"Expected a number after \"w=\"\n");
									n->eprint(mess);
									exit(0);
								}
								while ((*s >= '0') && (*s <= '9'))
								{
									wp = wp*10 + (*s-'0');
									s++;
								}
								s--;
							}
							n->SetWaitPeriod(wp);
							break;

					case 'g': case 'G':	/* Debug Switch */
							n->SetDebugSwitch(1);
							f = 1;
							if (s[1] == '=')
							{
								s += 2; /* point to number string */
								f=0;
								if ((*s < '0') || (*s > '9'))
								{
									char mess[256];
									sprintf  (mess,"Expected a number after \"g=\"\n");
									n->eprint(mess);
									exit(0);
								}
								while ((*s >= '0') && (*s <= '9'))
								{
									f = f*10 + (*s-'0');
									s++;
								}
								s--;
							}
							n->SetFactor(f);
							break;
					case 'x': case 'X': /* automatic matching switch off */
							n->SetAutoCCSwitch(0);
							break;

					case 'a': case 'A':
							n->SetSkipFPGA(1);
							break;

					case 't' : case 'T':
							t=1;
							break;

					case 'j' : case 'J':
							n->SetUseSlotSwitch(1);
							break;
#if _VXW_
						case 'm': case 'M': mm_switch = 1; /* Master Mode */ break;

						case 'n': case 'N': int_switch = 1; /* Interrupts */ break;

						case 'q': case 'Q': if (s[1] =='1')
							  {
								switch (s[2])
								{
									case '0': irq = 10; break;
									case '1': irq = 11; break;
									case '2': irq = 12; break;
									case '5': irq = 15; break;
									default :
										char mess[256];
										sprintf  (mess,"Invalid IRQ number (%c%c) after -q option\n", s[1],s[2]);
										n->eprint(mess);
										exit(0);
										break;
								}
								s += 2;
							  }
							  else
							  {
								  char mess[256];
								  sprintf  (mess,"Invalid IRQ number (%c%c) after -q option\n", s[1], s[2]);
								  n->eprint(mess);
								  exit(0);
							  }
						break;
#endif

						case '\0':	/* End of this argument pack. */
						break;

						default:
							char mess[256];
							sprintf  (mess,"%s: [%c] Unknown Option\n", argv[0], *s);
							n->eprint(mess);
							exit(0);
						break;
					}
				}
			break;

			default:	/* It is the name of the description file */
				if ((filename_ok == FALSE) && (s != NULL))
				{
					/* Filter eol character for LINUX/VXW fro cross platform shell/batch files */
					int l = strlen(s);
					if (s[l-1]==13) s[l-1]=0;
					if (s[l-1]==10) s[l-1]=0;
					filename_ok = TRUE;
					n->AddNetworkFileName(s);
				}
				else
				{
					char mess[256];
					sprintf  (mess,"\"%s\" or \"%s\"? Confusion in name of network file\n", n->GetNetworkFileName(), s);
					n->eprint(mess);
					exit(0);
				}
			break;
		}
	}

	if (filename_ok == FALSE)
	{
		char mess[256];
		sprintf  (mess,"No network file specified.\n");
		n->eprint(mess);
		exit(0);
	}

	if (t)
	{
		add_include_path(n, ".\\debug");
	}
	else
	{
		add_include_path(n, ".\\release");
	}

	if (n->GetVerboseSwitch())
	{
		char mess[256];

		ShowVersion();
		sprintf  (mess,"ResetSwitch        = %d\n", n->GetResetSwitch() );
		n->vprint(mess);
		sprintf  (mess,"LoadSwitch         = %d\n", n->GetLoadSwitch() );
		n->vprint(mess);
		sprintf  (mess,"ServeSwitch        = %d\n", n->GetServeSwitch() );
		n->vprint(mess);
		sprintf  (mess,"VerboseSwitch      = %d\n", n->GetVerboseSwitch());
		n->vprint(mess);
		sprintf  (mess,"FlashSwitch        = %d\n", n->GetFlashSwitch() );
		n->vprint(mess);
		sprintf  (mess,"DualSwitch         = %d\n", n->GetDualSwitch() );
		n->vprint(mess);
		sprintf  (mess,"DebugSwitch        = %d\n", n->GetDebugSwitch() );
		n->vprint(mess);
		if (n->GetDebugSwitch())
		{
			sprintf  (mess,"(factor is = %d (%d sec))\n", n->GetFactor(), n->GetFactor()*1000);
			n->vprint(mess);
		}
		sprintf  (mess,"ProgramFlashSwitch = %d\n", n->GetpgFlashSwitch() );
		n->vprint(mess);
		sprintf  (mess,"Wait Period        = %d\n", n->GetWaitPeriod() );
		n->vprint(mess);
		sprintf  (mess,"Network file       = \"%s\"\n", n->GetNetworkFileName() );
		n->vprint(mess);
		i = 0;
		s = n->GetIncludePath(i);
		while (s)
		{
			sprintf  (mess,"Scan directory %-2d  = \"%s\"\n", i, s);
			n->vprint(mess);
			s = n->GetIncludePath(++i);
		}
	}
}

int InCC = 0;

#if	_SUN_
void bus_error( int fg)
{
    fprintf( stderr, "Bus Error, exiting nicely...\n");
    exit( -1);
}
#endif
#if	_PC_
void catch_ctl_c( int a)
{
    fprintf( stderr, "Intercept CtlC , exiting nicely...\n");
#if (WIN32 && CCSTUDIO)
	if (InCC)
	{
		CoUninitialize();
	}
#endif
    exit( -1);
}
#endif
#if	_HEAPI_
void catch_ctl_c( int a)
{
    fprintf( stderr, "Intercept CtlC , exiting nicely...\n");
    exit( -1);
}
#endif

char	description_filename[256];	/* Nom du fichier de description */
char	spath[256];

void Mutilate(char *str)
{
	int l;

	l = strlen(str);
	while (l > 0)
	{
		if (str[l] != '\\') l--;
		else break;
	}
	str[l] = 0;
}

int DoServerLoader(int argc, char *argv[], char *brdname, int *boardno, int *device)
{
	int		r = 0;
	network	net;
	char   *devname;
#ifdef CCSTUDIO
	ccif    cc;
#endif

	parse_command_line(&net, argc, argv);


#if	_SUN_
	signal( SIGBUS, bus_error );
#endif
#if	_PC_ /*|| _HEAPI_ )*/
	signal( SIGINT, catch_ctl_c );
#endif

	if (net.GetpgFlashSwitch())
	{
		net.verify();
	}
	else
	{
	r = net.parse_input_file(NULL);	// Parse the network file. NULL as an argument
									// tells the function to use the network file
									// as deducted from the command line.
	if (r) return 0;

	if (net.GetDebugSwitch())
	{
#if (WIN32 && CCSTUDIO)

		// Find out the path to the network file. We will 
		// use spath to set Code Composer's search directory.
		if (description_filename[1] == ':')
		{
			strcpy  (spath, description_filename);
			Mutilate(spath);
		}
		else
		{
			GetCurrentDirectory(256, spath);
		}

		// Start up Code composer, 
		if (!cc.StartCodeComposer(spath)) return 0;
		
		InCC = 1;

		// Match Heron ID's with CC id's
		cc.MatchHeronId2CCid(&net);

		// and do a RunFree,
		if (cc.RunFree() > 0) return 0;

		// We need to give CC some time here to release 
		// the processors. It's a bit unclear how much time
		// is sufficient; and per machine it will differ.
		// How to solve this properly?
		Sleep(net.GetFactor() * 1000);


		if (r == 0) r = net.open  ();	// Open all the boards;
		if (r == 0) r = net.reset ();	// Reset all the boards.
		if (r == 0) r = net.load  ();	// load all the nodes on all the boards

		Sleep(net.GetFactor() * 1000);

		cc.PrepareDebugging(&net);



		Sleep(net.GetFactor() * 1000);

		// Now that the breakpoints were set, start the network

		if (r == 0) r = net.run   ();	// run all the nodes of the network		

		// Collect all the breakpoint hits of all tasks
		cc.OnHitBreakpoint(&net);

		cc.StartDebugging(&net);



		CoUninitialize();
		InCC = 0;


#if WANTSERVE
			if (r == 0)     net.serfit();
#endif

			// Now retrieve useful information for API part
			// These functions work ONLY with HEAPI entries in the network file.
			// E.g. they return what is found in a BD API ... line. The zero pa-
			// rameter in the functions denotes the first BD API... line. 
			 devname = net.GetDevName(0);
			*boardno = net.GetBoardNo(0);
			*device  = net.GetComport(0);

			strcpy(brdname, devname);
			net.close();

#else
			printf("Sorry, the '-g' switch is not supported in this version\n");
#endif
		}
		else
		{
			// No debugger wanted. Simply proceed Server/Loader.
			if (r == 0) r = net.open  ();	// Open all the boards;
			if (r == 0) r = net.reset ();	// Reset all the boards.
			if (r == 0) r = net.load  ();	// load all the nodes on all the boards
			if (r == 0) r = net.run   ();	// run all the nodes of the network		
#if WANTSERVE
			if (r == 0)     net.serfit();
#endif
			// Now retrieve useful information for API part
			// These functions work ONLY with HEAPI entries in the network file.
			// E.g. they return what is found in a BD API ... line. The zero pa-
			// rameter in the functions denotes the first BD API... line. 
			 devname = net.GetDevName(0);
			*boardno = net.GetBoardNo(0);
			*device  = net.GetComport(0);

			// Variable 'devname' is a pointer into the class object.
			// But the object will be close/destroyed at this function's completion.
			// Therefore copy the brdname out of the object.
			strcpy(brdname, devname);

			net.close();
		}
	}

	return 1;
}

int				bnumber;				// board number
char			*bname = NULL;			// board name (API compliant)
int				comport = 0;			// device (hep8a: only 0)
HE_DWORD		status;
HE_HANDLE		hDevice;
HE_IOSTATUS		ReadIoStatus;
HE_IOSTATUS		WriteIoStatus;
HE_MEMHANDLE	memHandle = NULL;
HE_PDWORDBUFFER test;

void api_read(HE_INT32 Count, unsigned char *Buffer)
{
	HE_DWORD	status;
    
    status = HeRead(hDevice, (void *)Buffer, Count, ReadIoStatus);
	if (status == HE_IoInProgress)
	{
        status = HeWaitForIo(hDevice, ReadIoStatus);
    }

	if (status != HE_OK)
	{
		char mess[256];
		HeErr2Text(status, mess);
		printf("Error reading board \"%s\" (%s).", bname, mess);
		exit(0);
    }
}

int api_write(HE_INT32 Count, unsigned char *Buffer)
{
	HE_DWORD status;

    status = HeWrite(hDevice, (void *)Buffer, Count, WriteIoStatus);
    if (status == HE_IoInProgress)
	{
        status = HeWaitForIo(hDevice, WriteIoStatus);
    }

	if (status != HE_OK)
	{
		char mess[256];
		HeErr2Text(status, mess);
		printf("Error writing board \"%s\" (%s).", bname, mess);
		exit(0);
    }

	return 0;
}

void DoApi(char *brdname, int bno, int device)
{
	hDevice = NULL;
	bname	= brdname;
	bnumber	= bno;
	comport = device;
	status  = HeOpen(bname, bnumber, device, &hDevice);
	if (status != HE_OK)
	{	
		char mess[256];
		HeErr2Text(status, mess);
		printf("Cannot open \"%s\", device %d (%s).", bname, device, mess);
		exit(0);
	}

    ReadIoStatus = NULL;
    status = HeInitIoStatus(hDevice, &ReadIoStatus);
	if (status != HE_OK)
	{
		char mess[256];
		HeErr2Text(status, mess);
		printf("Cannot init read structure on \"%s\" (%s).", bname, mess);
		exit(0);
    }

	// Initialise write status structure
    WriteIoStatus = NULL;
    status = HeInitIoStatus(hDevice, &WriteIoStatus);
	if (status != HE_OK)
	{
		char mess[256];
		HeErr2Text(status, mess);
		printf("Cannot init write structure on \"%s\" (%s).", bname, mess);
		exit(0);
    }

	status = HeAlloc(&memHandle, 1024);			// allocate memory
	if (status != HE_OK)
	{
		char mess[256];
		HeErr2Text(status, mess);
		printf("Cannot allocate memhandle for \"%s\" (%s).", bname, mess);
		exit(0);
    }

	status = HeLock(memHandle, &test);				// get the address
	if (status != HE_OK)
	{
		char mess[256];
		HeErr2Text(status, mess);
		printf("Cannot lock memhandle for \"%s\" (%s).", bname, mess);
		exit(0);
    }

	printf("Write word. This will make the config light flash.\n");
	test[0] = 0x11223344;
	api_write(4, (unsigned char *) test);

	printf("Read back the message.\n");

	api_read(4, (unsigned char *) test);
	printf("message received was %x\n", test[0]);

	status = HeClose(&hDevice);
	if (status != HE_OK )
	{
		char mess[256];
		HeErr2Text(status, mess);
		printf("Cannot close board \"%s\" (%s).", bname, mess);
	}
}

void main(int argc, char *argv[])
{
	char brdname[256];
	int  boardno;
	int  device;

#ifdef _RTOS32
	// Initialize the kernel and keyboard driver
	RTKernelInit(5);
	//   CLKSetTimerIntVal(10*1000); // 10 millisecond timer tick
	KBInit();
#endif

	if (!DoServerLoader(argc, argv, brdname, &boardno, &device))
	{
		printf("Error running a Server/Loader function. Aborting.\n");
#ifdef _RTOS32
		getch();
#endif
		exit(0);
	}

	printf("[Server/Loader returned board info: %s %d %d]\n", brdname, boardno, device);

	DoApi(brdname, boardno, device);

	printf("End.\n");
#ifdef _RTOS32
	printf("(Press a key to continue)\n");
	getch();
#endif
}
