/*
 * stdio.c
 * Basic standard i/o demo program.
 * Before running this program, take a look at the 'network' file, 
 * it contains information describing the actual hardware configuration
 * and should be updated before running the server.
 *
 */

#include <stdioc60.h>
#include <string.h>
#include <std.h>
#include <swi.h>

static char     MyString[128];
static int      Data[4096];

static FILE    *
efopen(char *name, char *mode)
{
FILE           *F;

    if ((F = fopen(name, mode)) == NULL)
	{
		fprintf(stderr, "Can't open '%s' in mode '%s'\n", name, mode);
		srv_exit(0);
	}
	return F;
}

/* DSP/BIOS treats main() as an initialisation routine. While main() */
/* is being executed, DSP/BIOS is initialised but not running: all   */
/* hardware and software interrupts are disabled. This is a good     */
/* place to run bootloader(), since no other threads/tasks will be   */
/* running; tasks or threads that may concurrently try to communica- */
/* with processors that have not yet been booted.                    */

main()
{
	/* 
	 * The next line insures that the program is properly loaded onto
	 * the root DSP. The bootloader() function will call an external
	 * function "init_io_functions". This function is implemented in 
	 * a file "stub.c" in this directory. Please include this file
	 * into your project, and make sure that the correct heron.h
	 * file is included in this "stub.c" file .
	 */
	bootloader();
}

maintask()
{
int             i, c, *ip;
FILE           *F;
float           fl;
double          db;

	printf("\nPRINT TEXT FILE USING fgetc / fputc\n\n");

	F = efopen("stdio.c", "r");
	while ((c = fgetc(F)) != EOF)
	{
		fputc(c, stdout);
	}
	fclose(F);

	printf("\nPRINT TEXT FILE USING fgets / fputs\n\n");

	F = efopen("stdio.c", "r");
	while (fgets(MyString, 128, F) != NULL)
		fputs(MyString, stderr);
	fclose(F);

	for (i = 0; i < 4096; i++)
		Data[i] = i;

	printf("\nWRITE 2048 INTEGERS USING fwrite\n\n");

	F = efopen("stdio.dat", "wb");
	printf("fwrite : %5d\n", fwrite(Data, 4, 2048, F));
	fclose(F);

	printf("\nAPPEND 2048 INTEGERS USING fwrite\n\n");

	F = efopen("stdio.dat", "ab");
	printf("fwrite : %5d\n", fwrite(Data + 2048, 4, 2048, F));
	fclose(F);

	for (i = 0; i < 4096; i++)
		Data[i] = -1;

	printf("\nREAD 4096 INTEGERS USING fread\n\n");

	F = efopen("stdio.dat", "rb");
	for (ip = Data, i = 1; i <= 4096; ip += i, i <<= 1)
	{
		printf("fread : %4d / %4d  ", fread(ip, 4, i, F), i, feof(F));
		printf("feof : %d\n", feof(F));
	}
	for (i = 0; i < 4096; i++)
	{
		if (Data[i] != i)
		{
			printf("Error at %d, read %d\n", i, Data[i]);
			exit(-1);
		}
	}

	fclose(F);
	printf("\nWRITE 64Kbytes BLOCK\n\n");

	F = efopen("stdio.dat", "wb");
	fwrite((void *) 0x03000000, 4, 0x004000, F);
	fclose(F);
	printf("Written 64 Kbytes \n");
	remove("stdio.dat");

	printf("\nSYSTEM test\n\n");
	printf("Trying \"dir \"\n");
	srv_system("dir");

	fl = 1.2e-4;
	printf("float  (1.2e-4): %f, %g\n", fl, fl);
	db = 2.4e-8;
	printf("double (2.4e-8): %f, %g\n", db, db);

	/*
	 * 'srv_exit()' will also exit from the server.
	 */
	srv_exit( 0);
}
/*** End Of File ***/
